@@ -31,20 +31,13 @@
  * SUCH DAMAGE.
  */
 
-#ifndef lint
-static char sccsid[] = "@(#)utilities.c	8.3 (Berkeley) 5/30/95";
-#endif /* not lint */
-
-#include <config.h>
+/* based on @(#)utilities.c	8.1 (Berkeley) 6/6/93 */
 
 #define	TELOPTS
 #define	TELCMDS
 #define	SLC_NAMES
 #include <arpa/telnet.h>
 #include <sys/types.h>
-#ifdef HAVE_SYS_SELECT_H
-#include <sys/select.h>
-#endif
 #include <sys/time.h>
 
 #include <ctype.h>
@@ -120,13 +113,15 @@ SetNetTrace(file)
     if (file  && (strcmp(file, "-") != 0)) {
 	NetTrace = fopen(file, "w");
 	if (NetTrace) {
-	    strcpy((char *)NetTraceFile, file);
+	    strncpy((char *)NetTraceFile, file, sizeof(NetTraceFile) - 1);
+	    NetTraceFile[sizeof(NetTraceFile) - 1] = '\0';
 	    return;
 	}
 	fprintf(stderr, "Cannot open %s.\n", file);
     }
     NetTrace = stdout;
-    strcpy((char *)NetTraceFile, "(standard output)");
+    strncpy((char *)NetTraceFile, "(standard output)", sizeof(NetTraceFile) - 1);
+    NetTraceFile[sizeof(NetTraceFile) - 1] = '\0';
 }
 
     void
@@ -139,7 +134,6 @@ Dump(direction, buffer, length)
 #   define min(x,y)	((x<y)? x:y)
     unsigned char *pThis;
     int offset;
-    extern pettydump;
 
     offset = 0;
 
@@ -444,12 +438,13 @@ printsub(direction, pointer, length)
 		    fprintf(NetTrace, "(partial suboption??\?)");
 		    break;
 		}
-		fprintf(NetTrace, "%s|%s",
+		fprintf(NetTrace, "%s|%s%s",
 			((pointer[3] & AUTH_WHO_MASK) == AUTH_WHO_CLIENT) ?
 			"CLIENT" : "SERVER",
 			((pointer[3] & AUTH_HOW_MASK) == AUTH_HOW_MUTUAL) ?
-			"MUTUAL" : "ONE-WAY");
-
+			"MUTUAL" : "ONE-WAY",
+			((pointer[3] & AUTH_ENCRYPT_MASK) == AUTH_ENCRYPT_ON) ?
+			"|ENCRYPT" : "");
 		auth_printsub(&pointer[1], length - 1, buf, sizeof(buf));
 		fprintf(NetTrace, "%s", buf);
 		break;
@@ -466,11 +461,13 @@ printsub(direction, pointer, length)
 			fprintf(NetTrace, "(partial suboption??\?)");
 			break;
 		    }
-		    fprintf(NetTrace, "%s|%s ",
+		    fprintf(NetTrace, "%s|%s%s ",
 			((pointer[i] & AUTH_WHO_MASK) == AUTH_WHO_CLIENT) ?
 							"CLIENT" : "SERVER",
 			((pointer[i] & AUTH_HOW_MASK) == AUTH_HOW_MUTUAL) ?
-							"MUTUAL" : "ONE-WAY");
+							"MUTUAL" : "ONE-WAY",
+			((pointer[i] & AUTH_ENCRYPT_MASK) == AUTH_ENCRYPT_ON) ?
+							"|ENCRYPT" : "");
 		    ++i;
 		}
 		break;
@@ -491,11 +488,11 @@ printsub(direction, pointer, length)
 	    break;
 #endif
 
-#if	defined(ENCRYPTION)
+#ifdef	ENCRYPTION
 	case TELOPT_ENCRYPT:
 	    fprintf(NetTrace, "ENCRYPT");
 	    if (length < 2) {
-		fprintf(NetTrace, " (empty suboption?)");
+		fprintf(NetTrace, " (empty suboption??\?)");
 		break;
 	    }
 	    switch (pointer[1]) {
@@ -520,7 +517,7 @@ printsub(direction, pointer, length)
 		fprintf(NetTrace, " %s ", (pointer[1] == ENCRYPT_IS) ?
 							"IS" : "REPLY");
 		if (length < 3) {
-		    fprintf(NetTrace, " (partial suboption?)");
+		    fprintf(NetTrace, " (partial suboption??\?)");
 		    break;
 		}
 		if (ENCTYPE_NAME_OK(pointer[2]))
@@ -560,7 +557,7 @@ printsub(direction, pointer, length)
 		break;
 	    }
 	    break;
-#endif
+#endif	/* ENCRYPTION */
 
 	case TELOPT_LINEMODE:
 	    fprintf(NetTrace, "LINEMODE ");
@@ -598,7 +595,7 @@ printsub(direction, pointer, length)
 		    break;
 		}
 		break;
-
+		
 	    case LM_SLC:
 		fprintf(NetTrace, "SLC");
 		for (i = 2; i < length - 2; i += 3) {
@@ -730,7 +727,7 @@ printsub(direction, pointer, length)
 			    fprintf(NetTrace, "\n");
 
 			break;
-
+				
 		    default:
 			fprintf(NetTrace, " %d", pointer[i]);
 			break;
@@ -824,7 +821,6 @@ printsub(direction, pointer, length)
 			    break;
 
 			default:
-			def_case:
 			    if (isprint(pointer[i]) && pointer[i] != '"') {
 				if (noquote) {
 				    putc('"', NetTrace);
