@@ -871,6 +871,10 @@ void log_badverf(gss_name_t client_name, gss_name_t server_name,
 	  {15, "GET_POLS"},
 	  {16, "SETKEY_PRINCIPAL"},
 	  {17, "SETV4KEY_PRINCIPAL"},
+	  {18, "CREATE_PRINCIPAL3"},
+	  {19, "CHPASS_PRINCIPAL3"},
+	  {20, "CHRAND_PRINCIPAL3"},
+	  {21, "SETKEY_PRINCIPAL3"}
      };
 #define NPROCNAMES (sizeof (proc_names) / sizeof (struct procnames))
      OM_uint32 minor;
