@@ -76,7 +76,7 @@
 #include <assert.h>
 
 /*
- * $Id: init_sec_context.c,v 1.51.2.8 2000/09/19 23:12:34 meeroh Exp $
+ * $Id: init_sec_context.c,v 1.51.2.8.2.3 2002/02/19 23:36:05 dalmeida Exp $
  */
 
 /* XXX This is for debugging only!!!  Should become a real bitfield
@@ -99,7 +99,6 @@ static krb5_error_code get_credentials(context, cred, server, now,
 {
     krb5_error_code	code;
     krb5_creds 		in_creds;
-    int i;
     
     memset((char *) &in_creds, 0, sizeof(krb5_creds));
 
@@ -111,31 +110,11 @@ static krb5_error_code get_credentials(context, cred, server, now,
 
     in_creds.keyblock.enctype = 0;
 
-    /*
-     * Initial iteration is necessary to catch a non-matching
-     * credential prior to looping through the GSSAPI-supported
-     * enctypes, since an enctype mismatch in the loop below will
-     * return KRB5_CC_NOTFOUND rather than one of the other error
-     * codes.
-     */
-    code = krb5_get_credentials(context, 0, cred->ccache,
-				&in_creds, out_creds);
+    code = krb5_set_default_tgs_enctypes (context, enctypes);
     if (code)
-	goto cleanup;
-    krb5_free_creds(context, *out_creds);
-    *out_creds = NULL;
-    for (i = 0; enctypes[i]; i++) {
-	in_creds.keyblock.enctype = enctypes[i];
-	code = krb5_get_credentials(context, 0, cred->ccache, 
-				    &in_creds, out_creds);
-	if (code != KRB5_CC_NOT_KTYPE && code != KRB5_CC_NOTFOUND
-	    && code != KRB5KDC_ERR_ETYPE_NOSUPP)
-	    break;
-    }
-    if (enctypes[i] == 0) {
-	code = KRB5_CONFIG_ETYPE_NOSUPP;
-	goto cleanup;
-    }
+      goto cleanup;
+        code = krb5_get_credentials(context, 0, cred->ccache,
+				&in_creds, out_creds);
     if (code)
 	goto cleanup;
 
@@ -350,15 +329,16 @@ krb5_gss_init_sec_context(minor_status, claimant_cred_handle,
 #endif
      ENCTYPE_DES_CBC_CRC,
      ENCTYPE_DES_CBC_MD5, ENCTYPE_DES_CBC_MD4,
-     0
    };
+#define N_WANTED_ENCTYPES (sizeof(wanted_enctypes)/sizeof(wanted_enctypes[0]))
+   krb5_enctype requested_enctypes[N_WANTED_ENCTYPES + 1];
+   krb5_enctype *default_enctypes = 0;
    krb5_error_code code; 
    krb5_gss_ctx_id_rec *ctx, *ctx_free;
    krb5_timestamp now;
    gss_buffer_desc token;
-   int i, j, err;
+   int i, j, k, err;
    int default_mech = 0;
-   krb5_ui_4 resp_flags;
    OM_uint32 major_status;
 
    if (GSS_ERROR(kg_get_context(minor_status, &context)))
@@ -483,8 +463,52 @@ krb5_gss_init_sec_context(minor_status, claimant_cred_handle,
 				      &ctx->there)))
 	  goto fail;
 
+      code = krb5_get_tgs_ktypes (context, 0, &default_enctypes);
+      if (code)
+	  goto fail;
+      /* "i" denotes *next* slot to fill.  Don't forget to save room
+	 for a trailing zero.  */
+      i = 0;
+      for (j = 0;
+	   (default_enctypes[j] != 0
+	    /* This part should be redundant, but let's be paranoid.  */
+	    && i < N_WANTED_ENCTYPES);
+	   j++) {
+
+	  int is_duplicate_enctype;
+	  int is_wanted_enctype;
+
+	  krb5_enctype e = default_enctypes[j];
+
+	  /* Is this enctype one of the ones we want for GSSAPI?  */
+	  is_wanted_enctype = 0;
+	  for (k = 0; k < N_WANTED_ENCTYPES; k++) {
+	      if (wanted_enctypes[k] == e) {
+		  is_wanted_enctype = 1;
+		  break;
+	      }
+	  }
+	  /* If unwanted, go to the next one. */
+	  if (!is_wanted_enctype)
+	      continue;
+
+	  /* Is this enctype already in the list of enctypes to
+	     request?  (Is it a duplicate?)  */
+	  is_duplicate_enctype = 0;
+	  for (k = 0; k < i; k++) {
+	      if (requested_enctypes[k] == e) {
+		  is_duplicate_enctype = 1;
+		  break;
+	      }
+	  }
+	  /* If it is not a duplicate, add it. */
+	  if (!is_duplicate_enctype)
+	      requested_enctypes[i++] = e;
+      }
+      requested_enctypes[i++] = 0;
+
       if ((code = get_credentials(context, cred, ctx->there, now,
-				  ctx->endtime, wanted_enctypes, &k_cred)))
+				  ctx->endtime, requested_enctypes, &k_cred)))
 	  goto fail;
 
       if (default_mech) {
@@ -624,7 +648,7 @@ krb5_gss_init_sec_context(minor_status, claimant_cred_handle,
    } else {
       unsigned char *ptr;
       char *sptr;
-      krb5_data ap_rep, mic;
+      krb5_data ap_rep;
       krb5_ap_rep_enc_part *ap_rep_data;
       krb5_error *krb_error;
 
