@@ -5,11 +5,12 @@ kadm5_ret_t kadm5_chpass_principal_util(void *server_handle,
 					krb5_principal princ,
 					char *new_pw, 
 					char **ret_pw,
-					char *msg_ret)
+					char *msg_ret,
+					int msg_len)
 {
   kadm5_server_handle_t handle = server_handle;
 
   CHECK_HANDLE(server_handle);
   return _kadm5_chpass_principal_util(handle, handle->lhandle, princ,
-				      new_pw, ret_pw, msg_ret);
+				      new_pw, ret_pw, msg_ret, msg_len);
 }
