@@ -20,9 +20,6 @@ krb5_error_code krb5int_initialize_library (void)
 	    add_error_table(&et_kv5m_error_table);
 	    add_error_table(&et_kdb5_error_table);
 	    add_error_table(&et_asn1_error_table);
-#ifdef macintosh
-		InstallSleepNotification ();
-#endif
 
 		initialized = 1;
 	}
@@ -42,9 +39,6 @@ void krb5int_cleanup_library (void)
 	krb5_stdcc_shutdown();
 #endif
 	
-#ifdef macintosh
-	RemoveSleepNotification ();
-#endif
 	remove_error_table(&et_krb5_error_table);
 	remove_error_table(&et_kv5m_error_table);
 	remove_error_table(&et_kdb5_error_table);
