@@ -12,7 +12,6 @@
 #include <stdio.h>
 #include <sys/types.h>
 #include <sys/file.h>
-#include <unistd.h>
 #include <signal.h>
 
 static char MORE[] = "more";
@@ -68,14 +67,13 @@ int ss_pager_create()
 void ss_page_stdin()
 {
 	int i;
-#ifndef NPOSIX
+#ifdef POSIX_SIGNALS
 	struct sigaction sa;
 	sigset_t mask;
 #endif
-	
 	for (i = 3; i < 32; i++)
 		(void) close(i);
-#ifndef NPOSIX
+#ifdef POSIX_SIGNALS
 	sa.sa_handler = SIG_DFL;
 	sigemptyset(&sa.sa_mask);
 	sa.sa_flags = 0;
@@ -84,12 +82,12 @@ void ss_page_stdin()
 	(void) signal(SIGINT, SIG_DFL);
 #endif
 	{
-#ifndef NPOSIX
+#ifdef POSIX_SIGNALS
 		sigemptyset(&mask);
 		sigaddset(&mask, SIGINT);
 		sigprocmask(SIG_UNBLOCK, &mask, (sigset_t *)0);
 #else
-		register int mask = sigblock(0);
+		int mask = sigblock(0);
 		mask &= ~sigmask(SIGINT);
 		sigsetmask(mask);
 #endif
